-- ---------------------------------------------------------------------------------
-- 
-- Converts the ivy system database from version 58 to version 59
-- 
-- ---------------------------------------------------------------------------------
-- 
-- This script was automatically generated. Do not edit it. Instead edit the source file
-- 
-- ---------------------------------------------------------------------------------
-- Database: Microsoft SQL Server
-- ---------------------------------------------------------------------------------
-- Copyright:
-- Axon Ivy AG, Baarerstrasse 12, 6300 Zug
-- ---------------------------------------------------------------------------------

-- Drop views which has a new definition or depend on changed tables
DROP VIEW IWA_TaskQuery
GO


-- Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_CaseDeleteTrigger
GO

-- Added columns of table IWA_Task
ALTER TABLE IWA_Task ADD
  BusinessCaseId BIGINT NULL
GO

-- Migrate businessCaseId to IWA_Task
UPDATE IWA_Task
SET IWA_Task.BusinessCaseId = (
    SELECT
        CASE
            WHEN IWA_Case.BusinessCaseId IS NOT NULL THEN IWA_Case.BusinessCaseId
            ELSE IWA_Case.CaseId
        END AS CaseId
    FROM IWA_Case
    WHERE IWA_Case.CaseId = IWA_Task.CaseId
)
GO

-- Make IWA_Task.BusinessCaseId NOT NULL
ALTER TABLE IWA_Task ALTER COLUMN
  BusinessCaseId BIGINT NOT NULL
GO


-- Create new indexes of table IWA_Task
CREATE INDEX IWA_Task_BusinessCaseIdIndex
ON IWA_Task (BusinessCaseId)
GO


-- Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_CaseDeleteTrigger
  ON IWA_Case FOR DELETE AS
  IF (@@ROWCOUNT > 0)
  BEGIN

    DELETE IWA_TaskSwitchEvent FROM IWA_TaskSwitchEvent, deleted
    WHERE IWA_TaskSwitchEvent.CaseId = deleted.CaseId

    UPDATE IWA_Case    SET IWA_Case.BusinessCaseId=NULL
    FROM IWA_Case, deleted
    WHERE IWA_Case.BusinessCaseId = deleted.CaseId

    DELETE IWA_Task FROM IWA_Task, deleted
    WHERE IWA_Task.CaseId = deleted.CaseId

    DELETE IWA_Task FROM IWA_Task, deleted
    WHERE IWA_Task.BusinessCaseId = deleted.CaseId

    DELETE IWA_WorkflowEvent FROM IWA_WorkflowEvent, deleted
    WHERE IWA_WorkflowEvent.CaseId = deleted.CaseId

    DELETE IWA_CaseMapEvent FROM IWA_CaseMapEvent, deleted
    WHERE IWA_CaseMapEvent.ProcessCaseId = deleted.CaseId

    DELETE IWA_CaseNote FROM IWA_CaseNote, deleted
    WHERE IWA_CaseNote.CaseId = deleted.CaseId

    END
GO


-- Recreate views which has a new definition or depend on changed tables
CREATE VIEW IWA_TaskQuery
(
  TaskId,
  CaseId,
  BusinessCaseId,
  ProcessModelId,
  ApplicationId,
  StartTaskSwitchEventId,
  EndTaskSwitchEventId,
  TaskStartId,
  TaskEndId,
  WorkerUserId,
  WorkerUserName,
  WorkerUserDisplayName,
  WorkerSessionId,
  ActivatorRoleId,
  ActivatorUserId,
  ActivatorName,
  ActivatorDisplayName,
  ExpiryActivatorRoleId,
  ExpiryActivatorUserId,
  ExpiryActivatorName,
  ExpiryActivatorDisplayName,
  ExpiryPriority,
  ExpiryTimestamp,
  ExpiryTaskStartElementPid,
  IsExpired,
  ExpiredCreatorTaskId,
  TimeoutedCreatorIntrmdtEventId,
  DelayTimestamp,
  "State",
  RequestPath,
  DisplayNameTemplate,
  Name,
  DisplayDescriptionTemplate,
  Description,
  Priority,
  StartTimestamp,
  EndTimestamp,
  BusinessCalendarId,
  WorkingTime,
  BusinessRuntime,
  FailedTimeoutTimestamp,
  NumberOfFailures,
  NumberOfResumes,
  Category,
  IsUpdatedOnStart,
  IsOffline,
  CurrentPriority,
  CurrentActivatorName,
  CurrentActivatorDisplayName,
  CurrentActivatorRoleId,
  CurrentActivatorUserId
)
AS
  SELECT
    IWA_Task.TaskId,
    IWA_Task.CaseId,
    IWA_Task.BusinessCaseId,
    IWA_Task.ProcessModelId,
    IWA_Task.ApplicationId,
    IWA_Task.StartTaskSwitchEventId,
    IWA_Task.EndTaskSwitchEventId,
    IWA_Task.TaskStartId,
    IWA_Task.TaskEndId,
    IWA_Task.WorkerUserId,
    IWA_Task.WorkerUserName,
    CASE WHEN WorkerUserId IS NOT NULL AND LEN(WorkerUser.FullName) > 0 THEN WorkerUser.FullName WHEN WorkerUserId IS NOT NULL THEN WorkerUser.Name ELSE NULL END,
    IWA_Task.WorkerSessionId,
    IWA_Task.ActivatorRoleId,
    IWA_Task.ActivatorUserId,
    IWA_Task.ActivatorName,
    CASE WHEN IWA_Task.ActivatorUserId IS NOT NULL AND LEN(ActivatorUser.FullName) > 0 THEN ActivatorUser.FullName WHEN IWA_Task.ActivatorUserId IS NOT NULL THEN ActivatorUser.Name WHEN IWA_Task.ActivatorRoleId IS NOT NULL AND LEN(ActivatorRole.DisplayNameTemplate) > 0 THEN ActivatorRole.DisplayNameTemplate WHEN IWA_Task.ActivatorRoleId IS NOT NULL THEN ActivatorRole.Name ELSE NULL END,
    IWA_Task.ExpiryActivatorRoleId,
    IWA_Task.ExpiryActivatorUserId,
    IWA_Task.ExpiryActivatorName,
    CASE WHEN IWA_Task.ExpiryActivatorUserId IS NOT NULL AND LEN(ExpiryActivatorUser.FullName) > 0 THEN ExpiryActivatorUser.FullName WHEN IWA_Task.ExpiryActivatorUserId IS NOT NULL THEN ExpiryActivatorUser.Name WHEN IWA_Task.ExpiryActivatorRoleId IS NOT NULL AND LEN(ExpiryActivatorRole.DisplayNameTemplate) > 0 THEN ExpiryActivatorRole.DisplayNameTemplate WHEN IWA_Task.ExpiryActivatorRoleId IS NOT NULL THEN ExpiryActivatorRole.Name ELSE NULL END,
    IWA_Task.ExpiryPriority,
    IWA_Task.ExpiryTimestamp,
    IWA_Task.ExpiryTaskStartElementPid,
    IWA_Task.IsExpired,
    IWA_Task.ExpiredCreatorTaskId,
    IWA_Task.TimeoutedCreatorIntrmdtEventId,
    IWA_Task.DelayTimestamp,
    IWA_Task."State",
    IWA_Task.RequestPath,
    IWA_Task.DisplayNameTemplate,
    IWA_Task.Name,
    IWA_Task.DisplayDescriptionTemplate,
    IWA_Task.Description,
    IWA_Task.Priority,
    IWA_Task.StartTimestamp,
    IWA_Task.EndTimestamp,
    IWA_Task.BusinessCalendarId,
    IWA_Task.WorkingTime,
    IWA_Task.BusinessRuntime,
    IWA_Task.FailedTimeoutTimestamp,
    IWA_Task.NumberOfFailures,
    IWA_Task.NumberOfResumes,
    IWA_Task.Category,
    IWA_Task.IsUpdatedOnStart,
    IWA_Task.IsOffline,
    CASE IWA_Task.IsExpired WHEN 1 THEN IWA_Task.ExpiryPriority WHEN 0 THEN IWA_Task.Priority END,
    CASE IWA_Task.IsExpired WHEN 1 THEN IWA_Task.ExpiryActivatorName WHEN 0 THEN IWA_Task.ActivatorName END,
    CASE WHEN IWA_Task.IsExpired = 1 AND IWA_Task.ExpiryActivatorUserId IS NOT NULL AND LEN(ExpiryActivatorUser.FullName) > 0 THEN ExpiryActivatorUser.FullName WHEN IWA_Task.IsExpired = 1 AND IWA_Task.ExpiryActivatorUserId IS NOT NULL THEN ExpiryActivatorUser.Name WHEN IWA_Task.IsExpired = 1 AND IWA_Task.ExpiryActivatorRoleId IS NOT NULL AND LEN(ExpiryActivatorRole.DisplayNameTemplate) > 0 THEN ExpiryActivatorRole.DisplayNameTemplate WHEN IWA_Task.IsExpired = 1 AND IWA_Task.ExpiryActivatorRoleId IS NOT NULL THEN ExpiryActivatorRole.Name WHEN IWA_Task.IsExpired = 0 AND IWA_Task.ActivatorUserId IS NOT NULL AND LEN(ActivatorUser.FullName) > 0 THEN ActivatorUser.FullName WHEN IWA_Task.IsExpired = 0 AND IWA_Task.ActivatorUserId IS NOT NULL THEN ActivatorUser.Name WHEN IWA_Task.IsExpired = 0 AND IWA_Task.ActivatorRoleId IS NOT NULL AND LEN(ActivatorRole.DisplayNameTemplate) > 0 THEN ActivatorRole.DisplayNameTemplate WHEN IWA_Task.IsExpired = 0 AND IWA_Task.ActivatorRoleId IS NOT NULL THEN ActivatorRole.Name ELSE NULL END,
    CASE IWA_Task.IsExpired WHEN 1 THEN IWA_Task.ExpiryActivatorRoleId WHEN 0 THEN IWA_Task.ActivatorRoleId END,
    CASE IWA_Task.IsExpired WHEN 1 THEN IWA_Task.ExpiryActivatorUserId WHEN 0 THEN IWA_Task.ActivatorUserId END
  FROM IWA_Task
    LEFT OUTER JOIN IWA_User AS ActivatorUser ON IWA_Task.ActivatorUserId = ActivatorUser.UserId
    LEFT OUTER JOIN IWA_Role AS ActivatorRole ON IWA_Task.ActivatorRoleId = ActivatorRole.RoleId
    LEFT OUTER JOIN IWA_User AS ExpiryActivatorUser ON IWA_Task.ExpiryActivatorUserId = ExpiryActivatorUser.UserId
    LEFT OUTER JOIN IWA_Role AS ExpiryActivatorRole ON IWA_Task.ExpiryActivatorRoleId = ExpiryActivatorRole.RoleId
    LEFT OUTER JOIN IWA_User AS WorkerUser ON IWA_Task.WorkerUserId = WorkerUser.UserId
GO

-- Delete process models that belong to the SYSTEM application
DELETE FROM IWA_ProcessModel WHERE ApplicationId=0
GO


-- 
-- Update Version
-- 
UPDATE IWA_Version SET Version=59
GO
