# SQL script to convert the database for MySQL from version 89 to version 90


# Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_UserDeleteTrigger;

# Added columns of table IWA_RoleProperty
ALTER TABLE IWA_RoleProperty ADD COLUMN
  SecurityMemberId VARCHAR(210);

# Create added foreign keys of table IWA_RoleProperty
ALTER TABLE IWA_RoleProperty ADD
(
 FOREIGN KEY (SecurityMemberId) REFERENCES IWA_Role(SecurityMemberId) ON DELETE CASCADE
);


# Added columns of table IWA_UserProperty
ALTER TABLE IWA_UserProperty ADD COLUMN
  SecurityMemberId VARCHAR(210);

# Create added foreign keys of table IWA_UserProperty
ALTER TABLE IWA_UserProperty ADD
(
 FOREIGN KEY (SecurityMemberId) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE
);


# Added columns of table IWA_UserAbsence
ALTER TABLE IWA_UserAbsence ADD COLUMN
  SecurityMemberId VARCHAR(210);

# Create added foreign keys of table IWA_UserAbsence
ALTER TABLE IWA_UserAbsence ADD
(
 FOREIGN KEY (SecurityMemberId) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE
);


# Added columns of table IWA_UserSubstitute
ALTER TABLE IWA_UserSubstitute ADD COLUMN
  SecurityMemberId VARCHAR(210);
ALTER TABLE IWA_UserSubstitute ADD COLUMN
  SubstituteUserSecurityMemberId VARCHAR(210);
ALTER TABLE IWA_UserSubstitute ADD COLUMN
  SubstituteRoleSecurityMemberId VARCHAR(210);

# Create added foreign keys of table IWA_UserSubstitute
ALTER TABLE IWA_UserSubstitute ADD
(
 FOREIGN KEY (SecurityMemberId) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE
);

ALTER TABLE IWA_UserSubstitute ADD
(
 FOREIGN KEY (SubstituteUserSecurityMemberId) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE
);

ALTER TABLE IWA_UserSubstitute ADD
(
 FOREIGN KEY (SubstituteRoleSecurityMemberId) REFERENCES IWA_Role(SecurityMemberId) ON DELETE CASCADE
);


# Added columns of table IWA_Note
ALTER TABLE IWA_Note ADD COLUMN
  SecurityMemberId VARCHAR(210);

# Create added foreign keys of table IWA_Note
ALTER TABLE IWA_Note ADD
(
 FOREIGN KEY (SecurityMemberId) REFERENCES IWA_User(SecurityMemberId) ON DELETE SET NULL
);


# Added columns of table IWA_UserLocation
ALTER TABLE IWA_UserLocation ADD COLUMN
  SecurityMemberId VARCHAR(210);

# Create added foreign keys of table IWA_UserLocation
ALTER TABLE IWA_UserLocation ADD
(
 FOREIGN KEY (SecurityMemberId) REFERENCES IWA_User(SecurityMemberId) ON DELETE CASCADE
);


# Dropped columns of table IWA_SecurityDescriptor
ALTER TABLE IWA_SecurityDescriptor DROP COLUMN OwnerUserId;


# Added columns of table IWA_AccessControl
ALTER TABLE IWA_AccessControl ADD COLUMN
  SecurityMemberId VARCHAR(210);

# Create added foreign keys of table IWA_AccessControl
ALTER TABLE IWA_AccessControl ADD
(
 FOREIGN KEY (SecurityMemberId) REFERENCES IWA_SecurityMember(SecurityMemberId) ON DELETE CASCADE
);


# Create new indexes of table IWA_UserProperty
CREATE PROCEDURE create_index_if_not_exists(sqlstatement VARCHAR(256))
BEGIN
  DECLARE already_exists CONDITION FOR SQLSTATE '23000';
  DECLARE columns_indexed CONDITION FOR SQLSTATE '42000';

  DECLARE EXIT HANDLER FOR already_exists, columns_indexed
  BEGIN
    -- Ignore exceptions for index already exists or columns already indexed
  END;

  SET @sqlstmt = sqlstatement;
  PREPARE stmt FROM @sqlstmt;
  EXECUTE stmt;
  DEALLOCATE PREPARE stmt;
END;
;

# Create new indexes of table IWA_UserProperty
CALL create_index_if_not_exists('CREATE INDEX IWA_UserProperty_SecurityMemberIdIndex ON IWA_UserProperty (SecurityMemberId);');

# Create new indexes of table IWA_UserAbsence
CALL create_index_if_not_exists('CREATE INDEX IWA_UserAbsence_SecurityMemberIdIndex ON IWA_UserAbsence (SecurityMemberId);');

# Create new indexes of table IWA_UserSubstitute
CALL create_index_if_not_exists('CREATE INDEX IWA_UserSubstitute_SecurityMemberIdIndex ON IWA_UserSubstitute (SecurityMemberId);');

CALL create_index_if_not_exists('CREATE INDEX IWA_UserSbs_SbsUserScrtMmbrIdIdx ON IWA_UserSubstitute (SubstituteUserSecurityMemberId);');

CALL create_index_if_not_exists('CREATE INDEX IWA_UserSbs_SbsRoleScrtMmbrIdIdx ON IWA_UserSubstitute (SubstituteRoleSecurityMemberId);');

# Create new indexes of table IWA_Note
CALL create_index_if_not_exists('CREATE INDEX IWA_Note_SecurityMemberIdIndex ON IWA_Note (SecurityMemberId);');

# Create new indexes of table IWA_UserLocation
CALL create_index_if_not_exists('CREATE INDEX IWA_UserLocation_SecurityMemberId ON IWA_UserLocation (SecurityMemberId);');

# Create new indexes of table IWA_AccessControl
CALL create_index_if_not_exists('CREATE INDEX IWA_AccessControl_SecMembrIdIdx ON IWA_AccessControl (SecurityMemberId);');
DROP PROCEDURE create_index_if_not_exists;

# Create new unique constraint of table IWA_Role
ALTER TABLE IWA_Role ADD UNIQUE (SecurityMemberId);


# Create new unique constraint of table IWA_User
ALTER TABLE IWA_User ADD UNIQUE (SecurityMemberId);


# Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_UserDeleteTrigger
AFTER DELETE ON IWA_User
FOR EACH ROW
BEGIN
  UPDATE IWA_Note
  SET UserId=NULL
  WHERE IWA_Note.UserId=OLD.UserId;

  DELETE FROM IWA_UserRole
  WHERE IWA_UserRole.UserId = OLD.UserId;

  DELETE FROM IWA_UserAbsence
  WHERE IWA_UserAbsence.UserId = OLD.UserId;

  DELETE FROM IWA_UserSubstitute
  WHERE IWA_UserSubstitute.UserId = OLD.UserId;

  DELETE FROM IWA_UserSubstitute
  WHERE IWA_UserSubstitute.SubstituteUserId = OLD.UserId;

  DELETE FROM IWA_UserProperty
  WHERE IWA_UserProperty.UserId = OLD.UserId;

  DELETE FROM IWA_AccessControl
  WHERE IWA_AccessControl.UserId = OLD.UserId;

  DELETE FROM IWA_UserLocation
  WHERE IWA_UserLocation.UserId = OLD.UserId;

END
;

# Drop removed indexes of table IWA_Role
DROP INDEX IWA_Role_SecurityMemberIdIndex
ON IWA_Role;

# Drop removed indexes of table IWA_User
DROP INDEX IWA_User_SecurityMemberIdIndex
ON IWA_User;

# Update Version
UPDATE IWA_Version SET Version=90;
