-- ---------------------------------------------------------------------------------
-- 
-- Converts the ivy system database from version 35 to version 36
-- 
-- ---------------------------------------------------------------------------------
-- 
-- This script was automatically generated. Do not edit it. Instead edit the source file
-- 
-- ---------------------------------------------------------------------------------
-- Database: Oracle
-- ---------------------------------------------------------------------------------
-- Copyright:
-- ivyTeam AG, Alpenstrasse 9, 6304 Zug
-- ---------------------------------------------------------------------------------


-- Drop trigger which depend on changed table(s)
DROP TRIGGER IWA_ApplicationDeleteTrg;

-- Create new added tables
CREATE TABLE IWA_TaskSignalEventReceiver
(
  TaskSignalEventId NUMBER(20) NOT NULL,
  WaitingTaskId NUMBER(20) NOT NULL REFERENCES IWA_Task(TaskId) ON DELETE CASCADE,
  SignalCodePattern VARCHAR2(200),
  SignaledTaskTaskStartId NUMBER(20) NOT NULL REFERENCES IWA_TaskStart(TaskStartId) ON DELETE CASCADE,
  StartedSignaledTaskId NUMBER(20),
  PRIMARY KEY (TaskSignalEventId)
)
TABLESPACE ${tablespaceName};

CREATE INDEX IWA_TskSigEvntRcvr_WtingTaskId
ON IWA_TaskSignalEventReceiver (WaitingTaskId)
TABLESPACE ${tablespaceName};

CREATE INDEX IWA_TskSigEvntRcvr_StrtSgTskId
ON IWA_TaskSignalEventReceiver (StartedSignaledTaskId)
TABLESPACE ${tablespaceName};

CREATE TABLE IWA_SignaledTask
(
  SignaledTaskId NUMBER(20) NOT NULL REFERENCES IWA_Task(TaskId) ON DELETE CASCADE,
  SignalCodePattern VARCHAR2(200),
  ReceivedSignalEventId NUMBER(20),
  PRIMARY KEY (SignaledTaskId)
)
TABLESPACE ${tablespaceName};

CREATE INDEX IWA_SigTask_RecvdSignalEventId
ON IWA_SignaledTask (ReceivedSignalEventId)
TABLESPACE ${tablespaceName};

CREATE TABLE IWA_SignalEvent
(
  SignalEventId NUMBER(20) NOT NULL,
  ApplicationId NUMBER(20) NOT NULL,
  SignalCode VARCHAR2(200) NOT NULL,
  SentTimestamp DATE NOT NULL,
  SentByUserName VARCHAR2(200),
  SentByUserId NUMBER(20) REFERENCES IWA_User(UserId) ON DELETE SET NULL,
  SentByTaskId NUMBER(20) REFERENCES IWA_Task(TaskId) ON DELETE SET NULL,
  SentByProcessElementPid VARCHAR2(200),
  PRIMARY KEY (SignalEventId)
)
TABLESPACE ${tablespaceName};

CREATE INDEX IWA_SignalEvent_SignalCode
ON IWA_SignalEvent (SignalCode)
TABLESPACE ${tablespaceName};

CREATE INDEX IWA_SignalEvent_SentTimestamp
ON IWA_SignalEvent (SentTimestamp)
TABLESPACE ${tablespaceName};

CREATE INDEX IWA_SignalEvent_SentByTaskId
ON IWA_SignalEvent (SentByTaskId)
TABLESPACE ${tablespaceName};

CREATE INDEX IWA_SignalEvent_SentByUserId
ON IWA_SignalEvent (SentByUserId)
TABLESPACE ${tablespaceName};

CREATE TABLE IWA_SignalEventData
(
  SignalEventId NUMBER(20) NOT NULL REFERENCES IWA_SignalEvent(SignalEventId) ON DELETE CASCADE,
  SignalData BLOB NOT NULL,
  PRIMARY KEY (SignalEventId)
)
TABLESPACE ${tablespaceName};

CREATE TABLE IWA_StartSignalEventElement
(
  TaskStartId NUMBER(20) NOT NULL REFERENCES IWA_TaskStart(TaskStartId) ON DELETE CASCADE,
  SignalCodePattern VARCHAR2(200),
  PRIMARY KEY (TaskStartId)
)
TABLESPACE ${tablespaceName};

CREATE INDEX IWA_StrtSigEvntElmnt_SgCdPttrn
ON IWA_StartSignalEventElement (SignalCodePattern)
TABLESPACE ${tablespaceName};

ALTER TABLE IWA_TaskSignalEventReceiver ADD
(
 FOREIGN KEY (StartedSignaledTaskId) REFERENCES IWA_SignaledTask(SignaledTaskId) ON DELETE SET NULL
);

ALTER TABLE IWA_SignaledTask ADD
(
 FOREIGN KEY (ReceivedSignalEventId) REFERENCES IWA_SignalEvent(SignalEventId) ON DELETE SET NULL
);


-- Drop unique constraint which depend on changed columns or was deleted
ALTER TABLE IWA_Library DROP UNIQUE (ApplicationId, Id, Version) DROP INDEX;

-- Changed columns of table IWA_Library
ALTER TABLE IWA_Library ADD
  Version_temp VARCHAR2(50) DEFAULT ' ' NOT NULL;
UPDATE IWA_Library
SET IWA_Library.Version_temp=to_char(IWA_Library.Version);
ALTER TABLE IWA_Library DROP COLUMN Version;
ALTER TABLE IWA_Library RENAME COLUMN Version_temp TO Version;


-- Create unique constraints which depend on changed columns
ALTER TABLE IWA_Library ADD UNIQUE (ApplicationId, Id, Version);


-- Changed columns of table IWA_LibraryVersionSpec
ALTER TABLE IWA_LibraryVersionSpec ADD
  Version_temp VARCHAR2(50) DEFAULT ' ' NOT NULL;
UPDATE IWA_LibraryVersionSpec
SET IWA_LibraryVersionSpec.Version_temp=to_char(IWA_LibraryVersionSpec.Version);
ALTER TABLE IWA_LibraryVersionSpec DROP COLUMN Version;
ALTER TABLE IWA_LibraryVersionSpec RENAME COLUMN Version_temp TO Version;



-- Create new indexes of table IWA_UserLocation
CREATE INDEX IWA_UserLocation_UserId
ON IWA_UserLocation (UserId)
TABLESPACE ${tablespaceName};


-- Create new indexes of table IWA_TaskLocation
CREATE INDEX IWA_TaskLocation_TaskId
ON IWA_TaskLocation (TaskId)
TABLESPACE ${tablespaceName};


-- Recreate trigger which depend on changed table(s)
CREATE TRIGGER IWA_ApplicationDeleteTrg
AFTER DELETE ON IWA_Application
FOR EACH ROW
BEGIN
  DELETE IWA_ProcessModel
  WHERE IWA_ProcessModel.ApplicationId = :old.ApplicationId;

  DELETE IWA_Library
  WHERE IWA_Library.ApplicationId = :old.ApplicationId;

  DELETE IWA_Role
  WHERE IWA_Role.ApplicationId = :old.ApplicationId;

  DELETE IWA_User
  WHERE IWA_User.ApplicationId = :old.ApplicationId;

  DELETE IWA_SignalEvent
  WHERE IWA_SignalEvent.ApplicationId = :old.ApplicationId;

  DELETE IWA_ContentManagementSystem
  WHERE IWA_ContentManagementSystem.ApplicationId = :old.ApplicationId;

  DELETE IWA_SecurityDescriptor
  WHERE IWA_SecurityDescriptor.SecurityDescriptorId = :old.SecurityDescriptorId;

END;
;


-- Delete process models that belong to the SYSTEM application
DELETE FROM IWA_ProcessModel WHERE ApplicationId=0;


-- 
-- Update Version
-- 
UPDATE IWA_Version SET Version=36;
